@@ -8,7 +8,7 @@ plot.eCAR <- function(x,...) {
   if(model == "Gaussian") b0 <- rep(0, length(omega))
   graphics::matplot(omega, cbind(beta.mn, beta.q025, beta.q975, b0),
                    type="l", lty=c(1,2,2,3), col=c(1,1,1,2),
-                   xlab=expression(omega), ylab=expression(beta(omega[k])), ...)
+                   xlab=expression(omega[k]), ylab=expression(beta(omega[k])), ...)
 
 }
 
